package Box;
#
# phroggy.com 5.3
#
# (C) 2004 Andy Lyttle, all rights reserved.
# This code may be distributed and modified under the terms of
# the GNU General Public License.
# http://phroggy.com/opensource/
# 

use strict;
use warnings;
use PhroggyIO;
use Paths;
use Localize;

our $GeneratedCount=0;

sub new {
  # Arguments:  hash of Title, Subtitle or numeric Date (optional), Class (optional),
  # Body, Success (boolean, optional), BoxID (optional)
  
  my($pkg,%prophash)=@_;
  $prophash{'Success'}=2 unless(exists $prophash{'Success'}); # um, what does this do?
  my $properties=bless \%prophash, $pkg;
  return $properties;
}

sub load {
  # Arguments: filename without path or extension, boolean don't translate?
  # Returns: array of Box objects
  
  # Boxes must start with WINDOW:Title and may optionally end with .
  
  my($pkg,$filename,$notrans)=@_;

  my @boxlist=();
  my %info=();
  my $data=ReadFile("$Paths::File{'docs'}/$filename.box",0,$notrans);
  $filename=~s/\//_/g; # change / to _ for ID
  if($data) {
    foreach my $line (split /\n/,$data) {
      if($line=~/^WINDOW:(.*)$/) {
        # start of a new window
        if(exists $info{'Title'}) {
          # just finished reading an old window, so push it onto the list
          $info{'BoxID'}=$filename.(@boxlist?'-'.(@boxlist+1):'');
          push(@boxlist,$pkg->new(%info));
        }
        %info=('Title'=>$1,'Success'=>1);
      } elsif($line=~/^SUBTITLE:(.*)$/) {
        $info{'Subtitle'}=$1;
      } elsif($line=~/^DATE:(.*)$/) {
        $info{'Date'}=DateToNum($1); # will set to 0 if not valid format
      } elsif($line=~/^CLASS:(.*)$/) {
        $info{'Class'}=$1;
      } elsif($line ne '.') {
        # ignore lines containing only . for legacy support
        $info{'Body'}.=$line."\n";
      }
    }
    if(exists $info{'Title'}) {
      # just finished reading the last window, so push it onto the list
      $info{'BoxID'}=$filename.(@boxlist?'-'.(@boxlist+1):'');
      push(@boxlist,$pkg->new(%info));
    }
  } else {
    # file not found
    push(@boxlist,$pkg->error("<p>An error occurred while trying to load this box.  Please check back later.</p>",$filename));
  }
  return(@boxlist);
}

sub Success {
  # Returns:  whether this box was loaded successfully or not
  # 0=no, 1=yes, 2=not loaded from a .box file
  
  my($self)=@_;
  return($self->{'Success'});
}

sub Info {
  # Returns:  title, subtitle
  
  my($self)=@_;
  return($self->{'Title'},$self->{'Subtitle'});
}

sub Title {
  # Returns:  title
  
  my($self)=@_;
  return $self->{'Title'};
}

sub Date {
  # Returns:  numeric date, or 0 if no date
  
  my($self)=@_;
  return $self->{'Date'}?$self->{'Date'}:0;
}

sub Text {
  # Returns:  body
  
  my($self)=@_;
  return $self->{'Body'};
}

sub error {
  # Arguments: text of error message, box id
  # Returns: Box object
  
  my($pkg,$text,$id)=@_;
  my %info=('Title'=>'ERROR',
            'Class'=>'AlertBox',
            'Body'=>"$text\n",
            'Success'=>0,
            'BoxID'=>$id);
  return $pkg->new(%info);
}

sub warning {
  # Arguments: text of error message
  # Returns: Box object
  
  my($pkg,$text,$id)=@_;
  my %info=('Title'=>'WARNING',
            'Class'=>'WarningBox',
            'Body'=>"$text\n",
            'BoxID'=>'warning-'.int(rand(100000)));
  return $pkg->new(%info);
}

##################################################

package DocBox;
our @ISA=('Box');
use PhroggyIO;
use Theme;
use Localize;

sub Render {
  my($self)=@_;
  
  my $title=$self->{'Title'};
  my $subtitle=LongDate($self->{'Date'}); # empty if not valid
  $subtitle=$self->{'Subtitle'} if($self->{'Subtitle'}); # override date with this
  my $class=$self->{'Class'}?$self->{'Class'}:'NormalBox';
  my $body=$self->{'Body'};
  my $boxid=$self->{'BoxID'}?$self->{'BoxID'}:('generated-'.++$GeneratedCount);

  my $html=$main::Theme->GetTemplate($subtitle?'Subtitle.tmpl':'Window.tmpl');
  $html=~s/{BOXCLASS}/DocBox $class/g;
  $html=~s/{WINDOWTITLE}/$title/g;
  $html=~s/{SUBTITLE}/$subtitle/g;
  $html=~s/{BOXID}/$boxid/g;
  $html=~s/{CONTENT}/$body/;
  return $html;
}

##################################################

package MenuBox;
our @ISA=('Box');
use PhroggyIO;
use Theme;

sub Render {
  my($self)=@_;

  my $title=$self->{'Title'};
  my $body=$self->{'Body'};
  my $boxid=$self->{'BoxID'}?$self->{'BoxID'}:('generated-'.++$GeneratedCount);
  my $class=$self->{'Class'}?$self->{'Class'}:'NormalBox';

  my $html=$main::Theme->GetTemplate('Menu.tmpl');
  
  $html=~s/{WINDOWTITLE}/$title/g;
  $html=~s/{BOXID}/$boxid/g;
  $html=~s/{BOXCLASS}/MenuBox $class/g;
  $html=~s/{CONTENT}/$body/;
  return $html;
}
