package Browser;
#
# phroggy.com 5.1
#
# (C) 2004 Andy Lyttle, all rights reserved.
# This code may be distributed and modified under the terms of
# the GNU General Public License.
# http://phroggy.com/opensource/
# 
# Functions to interact with environment variables and other
# data passed via the CGI interface.
#
# Public globals in this package:
# %Query is a hash of keys/values from the query string
# %Cookies is a hash of keys/values saved in cookies
# $BrowserLevel is an integer 2 or higher, vaguely related to how good the browser is
#

BEGIN {
  use base 'Exporter';
  our @EXPORT=qw(
    GetPost
  );
}

use strict;
use warnings;
use Encode;
use PhroggyIO;

our(%Query,%Cookies,$BrowserLevel)=();

_GetBrowserLevel();
_GetQuery();
_GetCookies();

sub GetPost {
  # Get form post data
  # Returns:  hash of values
  
  my %data=();
  return %data unless($ENV{'CONTENT_LENGTH'});
  read(STDIN, my $post, $ENV{'CONTENT_LENGTH'});

#  # DEBUG
#  my @time=localtime;
#  my $stamp=sprintf('%02u%02u%02u-%02u%02u%02u-%u',$time[5]-100,$time[4]+1,@time[3,2,1,0],$$);
#  if(open TMP,">/tmp/phroggy-DEBUG-$stamp.txt") {
#    print TMP $post;
#    close TMP;
#  }

  my @pairs=split(/&/, $post);
  foreach my $pair (@pairs) {
    my($name,$value)=split('=',$pair);
    $name=~tr/+/ /;
    $name=~s/%([a-f\d]{2})/chr hex($1)/ieg;
    $value=~tr/+/ /;
    $value=~s/%([a-f\d]{2})/chr hex($1)/ieg;
    $data{$name}=decode('utf-8',$value);
  }
  return %data;
}


sub _GetBrowserLevel {
  # Detect the level of the user's browser, and store in $BrowserLevel
  
  # Level 4 browsers support tables with background images or colors.
  # Level 3 browsers support tables with background colors, but not images.
  # Level 2 browsers support tables, but not with background images or colors.

  $BrowserLevel=2;
  if($ENV{'HTTP_USER_AGENT'}) {
    if($ENV{'HTTP_USER_AGENT'}=~/Mozilla\/(...)/) {
      $BrowserLevel=3 if($1>=3);
      $BrowserLevel=4 if($1>=4);
      $BrowserLevel=5 if($1>=5);
    }
    $BrowserLevel=5 if($ENV{'HTTP_USER_AGENT'}=~/OmniWeb\/(\d)/ && $1>3);
    $BrowserLevel=5 if($ENV{'HTTP_USER_AGENT'}=~/Opera\/(\d)/ && $1>3);
    $BrowserLevel=4 if($ENV{'HTTP_USER_AGENT'}=~/iCab/);
    $BrowserLevel=5 if($ENV{'HTTP_USER_AGENT'}=~/Konqueror/);
    $BrowserLevel=4 if($ENV{'HTTP_USER_AGENT'}=~/Konqueror\/2/);
    $BrowserLevel=3 if($ENV{'HTTP_USER_AGENT'}=~/Konqueror\/1/);
    $BrowserLevel=3 if($ENV{'HTTP_USER_AGENT'}=~/^Links /);
  }
}

sub _GetQuery {
  # Read the query string, and store the values in %Query

  my @pairs=(defined $ENV{'QUERY_STRING'})?split(/&/, $ENV{'QUERY_STRING'}):();
  foreach my $pair (@pairs) {
    next unless($pair=~/^([^=]+)=([^=]*)$/);
    my($name,$value)=($1,$2);
    $name=~tr/+/ /;
    $name=~s/%([a-f\d]{2})/chr hex($1)/ieg;
    next if($name=~/[^a-z]/); # ignore any key names with other characters
    $value=~tr/+/ /;
    $value=~s/%([a-f\d]{2})/chr hex($1)/ieg;
    $Query{$name}=$value;
  }
}

sub _GetCookies {
  # Read any cookies sent by the browser, and store them in %Cookies.
  # Cookie names must only contain lower-case letters.
  # Values must only contain letters, numbers, hyphens and underscores.
  
  %Cookies=();
  my $string=(defined $ENV{'HTTP_COOKIE'})?$ENV{'HTTP_COOKIE'}:'';
  my @pairs=split(/; /, $string);
  foreach my $pair (@pairs) {
    my($name,$value)=split(/=/,$pair);
    $name=~tr/+/ /;
    $name=~s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
    next if($name=~/[^a-z]/); # ignore any key names with other characters
    $value=~tr/+/ /;
    $value=~s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;

    unless($value=~/[^a-zA-Z0-9\-_]/) {
      $Cookies{$name}=$value;
    }
  }
}

1;
