package Dynamic;
#
# phroggy.com 5.1
#
# (C) 2004 Andy Lyttle, all rights reserved.
# This code may be distributed and modified under the terms of
# the GNU General Public License.
# http://phroggy.com/opensource/
# 
# These functions return dynamically-generated content.
#

BEGIN {
  use base 'Exporter';
  our @EXPORT=qw(
    GetDynamicMenu
    GetWelcomeBox
    GetSpamTrap
  );
}

use strict;
use warnings;
use File::Basename;
use Paths;
use Localize;
use PhroggyIO;
use Navigation;
use Themes;

sub GetDynamicMenu {
  # Arguments:  name of dynamic menu, filename of current page
  # Returns:  MenuBox
  
  my($name,$filename)=@_;
  if($name eq 'navigation') {
    return GetNavigationBox($filename);
  } elsif($name eq 'navigation-simple') {
    return GetNavigationSimpleBox($filename);
  } elsif($name eq 'themes') {
    return GetThemesBox($filename);
  } elsif($name eq 'themes-simple') {
    return GetThemesSimpleBox($filename);
  } elsif($name eq 'quote') {
    return GetQuoteBox($filename);
  } else {
    return error MenuBox('<p>No such menu.</p>');
  }
}

sub GetNavigationBox {
  my($filename)=@_;
  my @links=(
    {'filename'=>'home','icon'=>'home.gif','width'=>41},
    {'filename'=>'weblog','icon'=>'folder-3.gif','width'=>41},
    {'filename'=>'test','icon'=>'file.gif','width'=>41},
  );
  my $html="<div class=\"nav\">\n";
  foreach my $ref (@links) {
    if($filename && $$ref{'filename'} eq $filename) {
      $html.="\t\t".'<img src="'.$Paths::Web{'themes'}.'/'.$main::Theme->Name.'/nav/'.$$ref{'icon'};
      $html.='" width='.$$ref{'width'}.' height=18 border=0 align="top" alt="">';
      $html.=GetShortTitle($$ref{'filename'})."<br>\n";
    } else {
      $html.="\t".'<a href="'.GetURI($$ref{'filename'}).'" title="';
      $html.=GetDescription($$ref{'filename'}).'">'."\n";
      $html.="\t\t".'<img src="'.$Paths::Web{'themes'}.'/'.$main::Theme->Name.'/nav/'.$$ref{'icon'};
      $html.='" width='.$$ref{'width'}.' height=18 border=0 align="top" alt="">';
      $html.=GetShortTitle($$ref{'filename'})."</a><br>\n";
    }
  }
  $html.="</div>\n";
  return new MenuBox('Title'=>Translate('Navigation'),'BoxID'=>'NavigationMenu','Body'=>$html);
}

sub GetNavigationSimpleBox {
  my($filename)=@_;
  my @links=(
    'home',
    'weblog',
    'test',
  );
  my $rows=1; # change this as needed
  my $cols=int(@links/$rows);
  my $html='';
  for(my $row=0;$row<$rows;$row++) {
    my @thisrow=();
    if($row==$rows-1) { # this is last row
      @thisrow=@links;
    } else { # this is not last row
      @thisrow=@links[0..($cols-1)];
      splice @links,0,$cols;
    }
    my @html=();
    foreach my $link (@thisrow) {
      if($filename && $link eq $filename) {
        push @html,GetShortTitle($link);
      } else {
        push @html,LinkTo($link).GetShortTitle($link).'</a>';
      }
    }
    $html.='[ '.join(" |\n",@html)." ]<br>\n";
  }
  return new MenuBox('Title'=>Translate('Navigation'),'BoxID'=>'NavigationMenu','Body'=>$html);
}

sub GetThemesBox {
  my($filename)=@_;
  my %hash=GetThemesCache();
  my %themes=();
  my %bg=();
  my %ex=();
  foreach my $theme (keys %hash) {
    my @foo=@{$hash{$theme}};
    $bg{$theme}=shift @foo;
    $ex{$theme}=(shift @foo) eq 'true'?1:0;
    my %langs=@foo;
    $themes{$theme}=$langs{ChooseLanguage(keys %langs)};
  }
  %hash=GetBGCache();
  my %backgrounds=();
  foreach my $bg (keys %hash) {
    my @foo=@{$hash{$bg}};
    my %langs=@foo;
    $backgrounds{$bg}=$langs{ChooseLanguage(keys %langs)};
  }

  my $html='<form name="ThemeForm" action="'.GetURI($filename).'" method="get">'."\n";
  # theme menu
  $html.=Translate('Theme').":<br>\n";
  $html.='<select name="theme" onChange="changeTheme(this.value);">'."\n";
  foreach my $theme (sort {$themes{$a} cmp $themes{$b}} keys %themes) {
    $html.="\t".'<option value="'.$theme.'"';
    $html.=' SELECTED' if($theme eq $main::Theme->Name);
    $html.=">$themes{$theme}\n";
  }
  $html.="</select><br><br>\n\n";
  # background menu
  $html.=Translate('Background').":<br>\n";
  $html.='<select name="background"';
  $html.=' DISABLED' if($main::Theme->Info('exclusivebg'));
  $html.=">\n";
  my %bgindex=();
  my $bgcount=-1;
  foreach my $bg (sort {$backgrounds{$a} cmp $backgrounds{$b}} keys %backgrounds) {
    $html.="\t".'<option value="'.$bg.'"';
    $html.=' SELECTED' if($bg eq $main::Background->Name);
    $html.=">$backgrounds{$bg}\n";
    $bgindex{$bg}=++$bgcount;
  }
  $html.="</select><br><br>\n\n";
  $html.='<input type="submit" value="'.Translate('Change').'">'."\n";
  $html.="</form>\n";

  # Build JavaScript
    my $js='<script type="text/javascript"><!--
	function changeTheme(newTheme) {
		var exclusive=false;
		';

  foreach my $theme (sort keys %themes) {
    $js.='if(newTheme=="'.$theme.'") {'."\n";
    $js.='			document.ThemeForm.background.selectedIndex='.$bgindex{$bg{$theme}}."; // $bg{$theme}\n";
    $js.="			exclusive=true;\n" if($ex{$theme});
    $js.='		} else ';
  }
  $js=substr($js,0,length($js)-6)."\n";
  $js.='		if(document.ThemeForm.background.disabled != exclusive) document.ThemeForm.background.disabled=exclusive;
	}
// --></script>
';

  return new MenuBox('Title'=>Translate('Themes'),'BoxID'=>'ThemesMenu','Body'=>$js.$html);
}

sub GetThemesSimpleBox {
  my($filename)=@_;
  my %hash=GetThemesCache();
  my %themes=();
  my %bg=();
  my %ex=();
  foreach my $theme (keys %hash) {
    my @foo=@{$hash{$theme}};
    $bg{$theme}=shift @foo;
    $ex{$theme}=(shift @foo) eq 'true'?1:0;
    my %langs=@foo;
    $themes{$theme}=$langs{ChooseLanguage(keys %langs)};
  }
  my @themes=sort {$themes{$a} cmp $themes{$b}} keys %hash;

  my $rows=1; # change this as needed
  my $cols=int(@themes/$rows);
  my $html='';
  for(my $row=0;$row<$rows;$row++) {
    my @thisrow=();
    if($row==$rows-1) { # this is last row
      @thisrow=@themes;
    } else { # this is not last row
      @thisrow=@themes[0..($cols-1)];
      splice @themes,0,$cols;
    }
    my @html=();
    foreach my $theme (@thisrow) {
      if($theme eq $main::Theme->Name) {
        push @html,$themes{$theme};
      } else {
        push @html,LinkTo($filename,$theme).$themes{$theme}.'</a>';
      }
    }
    $html.='[ '.join(" |\n",@html)." ]<br>\n";
  }
  return new MenuBox('Title'=>Translate('Themes'),'BoxID'=>'ThemesMenu','Body'=>$html);
}

sub GetQuoteBox {
  my($filename)=@_; # not used here
  my $html='<div class="quote">{QUOTE}</div>'."\n";
  return new MenuBox('Title'=>Translate('Random Quote'),'BoxID'=>'QuoteMenu','Body'=>$html);
}

sub GetWelcomeBox {
  my @links=qw(
    test    weblog
  );

  # split list into columns
  my @cols=([],[]); # set number of columns here
  my $col=-1;
  foreach my $link (@links) {
    $col++;
    $col=0 if($col>=@cols);
    push @{$cols[$col]},$link;
  }
  
  # Get list of localized welcome.gif filenames
  my @files=glob("$Paths::File{'base'}/graphics/welcome/welcome*.gif");
  foreach my $file (@files) {
    $file=basename($file); # strip path from filename
  }

  my $html='<table id="WelcomeTable" cellspacing=0>'."\n";
  $html.="\t".'<tr><td colspan='.@cols.' class="WelcomeTo"><img src="{BASEPATH}/graphics/welcome/';
  $html.=LocalizeFilename('welcome.gif',@files).'" alt="'.Translate('Welcome to').'"></td></tr>'."\n";
  $html.="\t".'<tr><td colspan='.@cols.' class="Title">phroggy.com/opensource</td></tr>'."\n";
  $html.="\t".'<tr><td colspan='.@cols.' class="Count">'.Translate('Counter').'{COUNT}.</td></tr>'."\n";
  for(my $i=0;$i<@{$cols[0]};$i++) {
    $html.="\t<tr>\n";
    for($col=0;$col<@cols;$col++) {
      $html.="\t\t<td class=\"Icon\">";
      $html.=LinkTo($cols[$col]->[$i]).'<img src="{THEMEPATH}/icons/'.GetIcon($cols[$col]->[$i]).'" alt=""></a>'."\n\t\t" if(exists $cols[$col]->[$i]);
      $html.="</td>\n";
    }
    $html.="\t</tr>\n";
    $html.="\t<tr>\n";
    for($col=0;$col<@cols;$col++) {
      $html.="\t\t<td class=\"Label\">";
      $html.=LinkTo($cols[$col]->[$i]).GetShortTitle($cols[$col]->[$i])."</a>\n\t\t" if(exists $cols[$col]->[$i]);
      $html.="</td>\n";
    }
    $html.="\t</tr>\n";
  }
  $html.="\t".'<tr><td colspan='.@cols.' class="quote">{QUOTE}</td></tr>'."\n";
  $html.="</table>\n";
  return new DocBox('Title'=>Translate('Welcome to').' phroggy.com/opensource',
      'Class'=>'WhiteBox','BoxID'=>'welcome','Body'=>$html);
}

sub GetSpamTrap {
  my $fqdn=$ENV{'REMOTE_HOST'};
  my $ip=$ENV{'REMOTE_ADDR'};
  my $html='<!-- Spam Trap: these addresses are dynamically generated from your IP address. -->
<!-- Web browsers should ignore them, but any spider crawling the Web looking -->
<!-- for e-mail addresses to send spam to will add these to its list. -->
<div style="display: none;">
';
  my(@domains,@addr)=();
  if(!$fqdn || $fqdn eq $ip) {
    push(@domains,$ip) if($ip);
  } else {
    my @parts=split /\./,$fqdn;
    for(my $x=0;$x<$#parts;$x++) {
      push(@domains,join('.',@parts[$x..$#parts]));
    }
  }
  foreach my $dom (@domains,'localhost') {
    push(@addr,'abuse@'.$dom,'postmaster@'.$dom);
  }
  push(@addr,qw(
      spam@uce.gov
      tosemail1@aol.com
      nospam@moago.org
      caspam@cadoj.samspade.org
      honeypot@bl.phroggy.com
      spam@tuxserver.ath.cx
      scally@devolution.com
  )); # Add other addresses here
  foreach my $addr (@addr) {
    $html.="\t<a href=\"mailto:$addr\"></a>\n";
  }
  $html.='</div>';
}

1;
