package Paths;
#
# phroggy.com 5.05
#
# (C) 2004 Andy Lyttle, all rights reserved.
# This code may be distributed and modified under the terms of
# the GNU General Public License.
# http://phroggy.com/opensource/
# 
# These variables contain paths to various directories.
#
# Public globals in this package:
# %Web contains URI paths to send to the browser
# %File contains local paths to be accessed internally
# (see comments below)
#

use strict;
use warnings;
use PhroggyIO;

our(%Web,%File);

# Determine paths to various directories.  These paths may be different
# on different servers, and this function can be customized as needed.
# %Web is the URI for that location, %File is the path on the
# filesystem.  No trailing slashes on anything (this is important!).
#
# root = part of URL before base, determined from HTTP_HOST, defaults to "http://phroggy.com"
# base = root of the whole web site, usually "/~phroggy" or ""
# data = path to the webdata directory
# docs = path to the webdocs directory
# themes = path to the themes directory
# bg = path to the themes/backgrounds directory
# layouts = path to the layouts directory
# cgi = path to CGI scripts such as mail.pl
# temp = Counter.pm will create lockfiles here

# Start with default values to fall back on
my $user=$ENV{'USER'}?$ENV{'USER'}:'phroggy';
my $home=$ENV{'HOME'}?$ENV{'HOME'}:"/home/$user";
$File{'base'}="$home/public_html";
$Web{'root'}=$ENV{'HTTP_HOST'}?"http://$ENV{'HTTP_HOST'}":'http://phroggy.com';
$Web{'base'}="/~$user";

# Detect a particular server, and set values appropriately
if($ENV{SCRIPT_FILENAME} && $ENV{SCRIPT_FILENAME}=~m!^/domains/phroggy.com!) {
  $home="/domains/phroggy.com";
  $File{'base'}="$home/www";
  $File{'data'}="$home/www/webdata";
  $File{'docs'}="$home/www/webdocs";
  $Web{'base'}='';
}
if(-e '/Users/phroggy') {
  $home='/Users/phroggy';
  $File{'base'}="$home/Sites";
  $Web{'base'}='/~phroggy';
}
if($ENV{SCRIPT_FILENAME} && $ENV{SCRIPT_FILENAME}=~m!^/Users/phroggy/Sites/phroggy.com-opensource!) {
  $home='/Users/phroggy/Sites/phroggy.com-opensource';
  $File{'base'}=$home;
  $Web{'base'}='/~phroggy/phroggy.com-opensource';
  $Web{'cgi'}=$Web{'base'};
}

# Set these, if not overridden above
$File{'data'}   ="$home/webdata"             unless(exists $File{'data'});
$File{'docs'}   ="$home/webdocs"             unless(exists $File{'docs'});
$File{'themes'} ="$File{'base'}/themes"      unless(exists $File{'themes'});
$File{'bg'}     ="$File{'base'}/backgrounds" unless(exists $File{'bg'});
$File{'layouts'}="$File{'base'}/layouts"     unless(exists $File{'layouts'});
$File{'temp'}   =$File{'data'}               unless(exists $File{'temp'});
$Web{'themes'}  ="$Web{'base'}/themes"       unless(exists $Web{'themes'});
$Web{'bg'}      ="$Web{'base'}/backgrounds"  unless(exists $Web{'bg'});
$Web{'layouts'} ="$Web{'base'}/layouts"      unless(exists $Web{'layouts'});
$Web{'cgi'}     ="$Web{'base'}/cgi-bin"      unless(exists $Web{'cgi'});

# Sanity check
foreach (qw(base data docs themes bg layouts temp)) {
  Abort("Configuration error in Paths.pm, \$File{'$_'} not defined") unless(exists $File{$_});
  Abort("Configutation error in Paths.pm, \$File{'$_'} has trailing slash") if($File{$_}=~m!/$!);
  Abort("Can't find $_ folder, should be $File{$_}") unless(-d $File{$_});
  Abort("Can't read $_ folder, should be $File{$_}") unless(-r $File{$_});
}
foreach (qw(data temp)) {
  Abort("Can't write to $_ folder, should be $File{$_}") unless(-w $File{$_});
}
foreach (qw(root base themes bg layouts cgi)) {
  Abort("Configuration error in Paths.pm, \$Web{'$_'} not defined") unless(exists $Web{$_});
  Abort("Configutation error in Paths.pm, \$Web{'$_'} has trailing slash") if($Web{$_}=~m!/$!);
}


1;
