package PhroggyIO;
#
# phroggy.com 5.05
#
# (C) 2004 Andy Lyttle, all rights reserved.
# This code may be distributed and modified under the terms of
# the GNU General Public License.
# http://phroggy.com/opensource/
# 
# Basic file I/O, error messages, and a date parsing function.
#

BEGIN {
  use base 'Exporter';
  our @EXPORT=qw(
    ReadFile
    Alert
    GetAlerts
    Abort
    DateToNum
  );
}

use strict;
use warnings;
use Time::Local;
use File::Basename;
use Paths;
use Localize;
use Box;

our(@Warnings,%FilesRead,%FilesContent)=();

sub ReadFile {
  # Arguments: path and filename, nocache boolean, notranslate boolean
  # Returns: contents of file, or undef if not found

  # If the specified file is cached, the cached contents will be returned.
  # Otherwise, read the specified file, cache the contents (unless the second
  # argument is true), and return the contents.
  
  my($path,$nocache,$notrans)=@_;
  
  my $file=undef;
  if($FilesRead{$path}) {
    $file=$FilesContent{$path}; # localization is cached too
  } else {
    if($notrans) {
      # do not localize filename
      if(open(READ,$path)) {
        $file=join('',<READ>);
        close(READ);
      }
    } else {
	  # get list of filenames in dir without path
	  my @list=();
	  my($p,$f)=(dirname($path),basename($path));
	  return undef unless(-d $p);
	  foreach my $local (glob "$p/*") {
		push @list,basename($local);
	  }
	  # choose a localized filename
	  my $local=LocalizeFilename($f,@list);
	  if($local && open(READ,$p.'/'.$local)) {
		$file=join('',<READ>);
		close(READ);
	  }
	}
    unless($nocache) {
      $FilesContent{$path}=$file;
      $FilesRead{$path}=1;
    }
  }
  return $file;
}

sub Alert {
  # Arguments:  message to be displayed

  my($msg)=@_;
  push(@Warnings,warning DocBox("<p>$msg</p>"));
}

sub GetAlerts {
  # Returns: list of DocBoxes
  return @Warnings;
}

sub Abort {
  # Arguments:  error message

  # Print the specified error message on a web page with some help, and abort

  my($error)=@_;
  print 'Content-type: text/html; charset=iso-8859-1

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
        "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
	<title>phroggy.com: Fatal Error</title>
	<style type="text/css"><!--
		body {
			background-color: #fff;
			color: #990000;
			font-family: monospace;
			white-space: pre;
		}
	--></style>
</head>
<body>ERROR

Sorry, an error has occurred:
'.$error.'

Please try again in a few minutes.  I may be in the middle of uploading
changes to my site.  If it still doesn\'t work after a few minutes, please
send me e-mail and let me know, at web<!-- antispam -->&#64;<!-- antispam
-->phroggy.com.

</body>
</html>
';
  exit;
}

sub DateToNum {
  # Arguments: date string mm/dd/yy or mm/dd/yyyy
  # Returns: numeric date, or 0 if can't parse

  my($string)=@_;
  if($string=~m!^(\d+)/(\d+)/(\d+)!) {
    my($m,$d,$y)=($1,$2,$3);
    $y+=2000 if($y<38); # assume two-digit years before 38 are 20xx
    $y+=1900 if($y<100); # years 38-99 are 19xx
    return timelocal(0,0,0,$d,$m-1,$y-1900);
  } else {
    return 0;
  }
}


1;
