package Theme;
#
# phroggy.com 5.07
#
# (C) 2004 Andy Lyttle, all rights reserved.
# This code may be distributed and modified under the terms of
# the GNU General Public License.
# http://phroggy.com/opensource/
# 

use strict;
use warnings;
use Paths;
use PhroggyIO;
use Background;
use Localize;

sub new {
  # Arguments:  theme name

  # Reads the specified theme from the themes directory, if it exists.

  my($pkg,$theme)=@_;
  my $path="$Paths::File{'themes'}/$theme";
  my %data=('exists'=>0,'name'=>$theme,'path'=>$path);
  if($theme=~/[^a-z0-9\-]/) {
    Abort('Tried to read a theme containing invalid characters.');
  }
  my $info=ReadFile("$path/theme.info",undef,'notrans');
  if($info) {
    $data{'exists'}=1;
    foreach my $line (split /\n/,$info) {
      $data{$1}=$2 if($line && $line=~/^(.*?)=(.*)$/);
    }
  }
  return bless \%data, $pkg;
}

sub Error {
  # Returns: undef if this theme is valid, or an error token if not

  my($self)=@_;
  my $error=undef;
  if(!$self->{'exists'}) {
    $error="Theme missing";
  } elsif(!$self->{'title'}) {
    $error="Theme error";
  } elsif(!_ValidLayout($self->{'layout'})) {
    $error="Theme bad layout";
  } elsif($self->{'exclusivebg'}) {
    my $bg=new Background($self->{'exclusivebg'});
    unless($bg->Valid()) {
      $error="Theme bad background";
    }
  }
  return $error;
}

sub Info {
  # Arguments:  attribute name
  # Returns:  value
  
  my($self,$name)=@_;
  return $self->{$name};
}

sub Name {
  # Returns: name of this theme
  
  my($self)=@_;
  return $self->{'name'};
}

sub Layout {
  # Returns the name of this theme's layout
  
  my($self)=@_;
  return $self->{'layout'};
}

sub Title {
  # Returns: title of this theme (localized)
  
  my($self)=@_;  
  my %langs=();
  foreach (keys %$self) {
    if(/^title(_.+)?$/) {
      $langs{($1?substr($1,1):'en')}=$self->{$_};
    }
  }
  return $langs{ChooseLanguage(keys %langs)};
}

sub GetTemplate {
  # Arguments: filename within theme dir
  # Returns: content of file
  
  my($self,$filename)=@_;
  return ReadFile("$self->{'path'}/$filename",undef,'notrans');
}

sub GetLayoutTemplate {
  # Arguments: filename within layout dir
  # Returns: content of file
  
  my($self,$filename)=@_;
  return ReadFile("$Paths::File{'layouts'}/$self->{'layout'}/$filename",undef,'notrans');
}  

sub GetTitleList {
  # Returns: hash of titles in each available language
  # For Themes::_BuildThemesCache()

  my($self)=@_;
  my @list=();
  foreach my $key (keys %{$self}) {
    if($key=~/^title(_.+)?$/) {
      push @list,($1?substr($1,1):'en'),$self->{$key};
    }
  }
  return @list;
}

###

sub _ValidLayout {
  # Arguments: layout
  # Returns: 1 if valid, 0 if invalid

  my($l)=@_;
  my $ok=0;
  unless(!$l || $l=~/[^a-z0-9\-]/) {
    $ok=1 if(-f "$Paths::File{'layouts'}/$l/Page.tmpl");
  }
  return $ok;
}


1;
